
* file of standard 6811 register declarations
*********************************************************************
* Control Registers

* All registers referred to REGBASE ($1000)-> X reg.

REGBASE	EQU	$1000

X_PORTA	EQU	$00	; Port A data register
X_RESV1	EQU	$01	; Reserved
X_PIOC	EQU	$02	; Parallel I/O Control register
X_PORTC	EQU	$03	; Port C latched data register
X_PORTB	EQU	$04	; Port B data register
X_PORTCL	EQU	$05	;
X_DDRC	EQU	$07	; Data Direction register for port C
X_PORTD   	EQU 	$08	; Port D data register
X_DDRD    	EQU	$09	; Data Direction register for port D
X_PORTE	EQU	$0A	; Port E data register
X_CFORC	EQU	$0B	; Timer Compare Force Register
X_OC1M	EQU	$0C	; Output Compare 1 Mask register
X_OC1D	EQU	$0D	; Output Compare 1 Data register

* Two-Byte Registers (High,Low -- Use Load & Store Double to access)
X_TCNT	EQU	$0E	; Timer Count Register
X_TIC1	EQU	$10	; Timer Input Capture register 1
X_TIC2	EQU	$12	; Timer Input Capture register 2
X_TIC3	EQU	$14	; Timer Input Capture register 3
X_TOC1	EQU	$16	; Timer Output Compare register 1
X_TOC2	EQU	$18	; Timer Output Compare register 2
X_TOC3	EQU	$1A	; Timer Output Compare register 3
X_TOC4	EQU	$1C	; Timer Output Compare register 4
X_TI4O5	EQU	$1E	; Timer Input compare 4 or Output compare 5 register

X_TCTL1	EQU	$20	; Timer Control register 1
X_TCTL2	EQU	$21	; Timer Control register 2
X_TMSK1	EQU	$22	; main Timer interrupt Mask register 1
X_TFLG1	EQU	$23	; main Timer interrupt Flag register 1
X_TMSK2	EQU	$24	; misc Timer interrupt Mask register 2
X_TFLG2	EQU	$25	; misc Timer interrupt Flag register 2
X_PACTL	EQU	$26	; Pulse Accumulator Control register
X_PACNT	EQU	$27	; Pulse Accumulator Count register
X_SPCR	EQU	$28	; SPI Control Register
X_SPSR	EQU	$29	; SPI Status Register
X_SPDR	EQU	$2A	; SPI Data Register
X_BAUD	EQU	$2B	; SCI Baud Rate Control Register
X_SCCR1	EQU	$2C	; SCI Control Register 1
X_SCCR2	EQU	$2D	; SCI Control Register 2
X_SCSR	EQU	$2E	; SCI Status Register
X_SCDR	EQU	$2F	; SCI Data Register
X_ADCTL	EQU	$30	; A/D Control/status Register
X_ADR1	EQU	$31	; A/D Result Register 1
X_ADR2	EQU	$32	; A/D Result Register 2
X_ADR3	EQU	$33	; A/D Result Register 3
X_ADR4	EQU	$34	; A/D Result Register 4
X_BPROT	EQU	$35	; Block Protect register
X_RESV2	EQU	$36	; Reserved
X_RESV3	EQU	$37	; Reserved
X_RESV4	EQU	$38	; Reserved
X_OPTION	EQU	$39	; system configuration Options
X_COPRST	EQU	$3A	; Arm/Reset COP timer circuitry
X_PPROG	EQU	$3B	; EEPROM Programming register
X_HPRIO	EQU	$3C	; Highest Priority Interrupt and misc.
X_INIT	EQU	$3D	; RAM and I/O Mapping Register
X_TEST1	EQU	$3E	; factory Test register
X_CONFIG	EQU	$3F	; Configuration Control Register


* Interrupt Vector locations
* All Interrupt vectors referred to INTBASE ($FF00 or $BF00) -> X reg.

X_SCIINT	EQU	$D6	; SCI serial system
X_SPIINT	EQU	$D8	; SPI serial system
X_PAIINT 	EQU	$DA	; Pulse Accumulator Input Edge
X_PAOVINT 	EQU	$DC	; Pulse Accumulator Overflow
X_TOINT	EQU	$DE	; Timer Overflow
X_TOC5INT	EQU	$E0	; Timer Output Compare 5
X_TOC4INT	EQU	$E2	; Timer Output Compare 4
X_TOC3INT	EQU	$E4	; Timer Output Compare 3
X_TOC2INT	EQU	$E6	; Timer Output Compare 2
X_TOC1INT	EQU	$E8	; Timer Output Compare 1
X_TIC3INT	EQU	$EA	; Timer Input Capture 3
X_TIC2INT	EQU	$EC	; Timer Input Capture 2
X_TIC1INT	EQU	$EE	; Timer Input Capture 1
X_RTIINT	EQU	$F0	; Real Time Interrupt
X_IRQINT	EQU	$F2	; IRQ External Interrupt
X_XIRQINT	EQU	$F4	; XIRQ External Interrupt
X_SWIINT	EQU	$F6	; Software Interrupt
X_BADOPINT 	EQU	$F8	; Illegal Opcode Trap Interrupt
X_NOCOPINT 	EQU	$FA	; COP Failure (Reset)
X_CMEINT	EQU	$FC	; COP Clock Monitor Fail (Reset)
X_RESETINT 	EQU	$FE	; RESET Interrupt
